const orm = new (require('./OpenRouteMapDistanceResolver.js'))()
const sd = new (require('./SimpleDistanceResolver.js'))()

class CustomerFinder {
/**
 * @param {Type of customerListProvider} customerListProvider - provider that would give list of customers
 * @param {Type of homeLocation} homeLocation - the location where event would take place
 * @param {Object} [options] - optional arguments
 * @param {Boolean} [options.noORM] - disable fetching distance via ORM
 * @param {Boolean} [options.fallbackOnInfinity] - disable falling back to simple calculation in case ORM couldn't find routes
 */
  constructor (customerListProvider, homeLocation, options) {
    this._customers = customerListProvider.getList()
    this._home = homeLocation
    options = options || {}
    this._noORM = options.noORM || false
    this._fallbackOnInfinity = options.fallbackOnInfinity || true
  }

  /**
   * Asynchronously fetches the distance between "home" and target customer location
   * unless noORM is true, it tries to fetch route length from ORM and falls back to
   * simple greater-circle length calculation
   * @param {Customer} customer - customer from data
   * @param {Boolean} noORM - disable fetching from ORM
   * @returns {Promise<Number>} Promise of a distance in meters
   */
  _getCustomerDistance (customer, noORM) {
    const dr = noORM ? sd : orm
    return dr.getDistance(this._home, [customer.longitude, customer.latitude])
      .then(distance => (distance === Number.POSITIVE_INFINITY && this._fallbackOnInfinity)
        ? this._getCustomerDistance(customer, true)
        : distance)
    // probably should fall back for all following queries depending on response
      .catch(e => console.error(`[WARN] Failed fetching length via ${dr.constructor.name}: ${e.message}, falling back... to simple calculation`) ||
             this._getCustomerDistance(customer, true))
  }

  /**
   * Returns the list of customers within given limit
   * @param {Number} distanceLimit - maximum (exclusive) distance between event location and customer location
   * @returns {Promise<Customer[]>} filtered list of customers that match the distance criteria
   */
  findNearestCustomers (distanceLimit) {
    return this._customers
      .then(customers => customers
        .map(async customer => ((await this._getCustomerDistance(customer, this._noORM) < distanceLimit)) ? customer : undefined)
      )
      .then(promises => Promise.all(promises))
      .then(customers => customers.filter(_ => _))
  }
}

// just a test
// (new CustomerFinder(new (require('./CustomerListFromURL.js'))('https://s3.amazonaws.com/intercom-take-home-test/customers.txt'), [-6.257664, 53.339428], { noORM: false })).findNearestCustomers(100 * 1000).then(_ => console.log(_.length))
// this is what would be happening.

module.exports = CustomerFinder
